package com.codegeist.fisheye.internal.service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import com.codegeist.fisheye.domain.Changeset;
import com.codegeist.fisheye.domain.RevisionKey;

public class ChangesetHandler extends DefaultHandler implements Handler<Changeset> {
	private StringBuilder buffer = new StringBuilder();
	private Changeset result;
	private SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'");

	private String csid;
	private Date date;
	private String author;
	private String branch;
	private boolean sealed;
	private String log;
	private RevisionKey[] revisions;
	
	private Handler<List<RevisionKey>> revisionKeyArrayHandler;
	private boolean inRevisions = false;
	
	@Override
	public void startDocument() throws SAXException {
		csid = null;
		author = null;
		date = null;
		branch = null;
		sealed = false;
		log = null;
		revisions = new RevisionKey[0];
		revisionKeyArrayHandler = new ArrayHandler<RevisionKey>("revisionkey", new RevisionKeyHandler());
		inRevisions = false;
	}
	

	@Override
	public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {
		buffer.setLength(0);
		
		if (inRevisions) {
			revisionKeyArrayHandler.startElement(uri, localName, name, attributes);
		} else {
			if ("changeset".equals(localName)) {
				csid = StringRegistry.internChangeSetId(attributes.getValue("csid"));
				author = StringRegistry.internAuthor(attributes.getValue("author"));
				date = parseDate(attributes.getValue("date"));
				sealed = Boolean.valueOf(attributes.getValue("sealed"));
				branch = StringRegistry.internBranch(attributes.getValue("branch"));
			} if ("revisions".equals(localName)) {
				inRevisions = true;
				revisionKeyArrayHandler.startDocument();
			}
		}
	}
	
	@Override
	public void endElement(String uri, String localName, String name) throws SAXException {
		if (inRevisions) {
			if ("revisions".equals(localName)) {
				inRevisions = false;
				revisionKeyArrayHandler.endDocument();
				List<RevisionKey> revisionsList = revisionKeyArrayHandler.getResult();
				revisions = revisionsList.toArray(new RevisionKey[revisionsList.size()]);
			} else {
				revisionKeyArrayHandler.endElement(uri, localName, name);
			}
		} else {
			// TODO check that we see only one log element
			if ("log".equals(localName)) {
				log = buffer.toString();
			}
		}
		
		buffer.setLength(0);
	}
	
	
	@Override
	public void endDocument() throws SAXException {
		result = new Changeset(csid, date, author, branch, sealed, log, revisions);
	}
	
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		if (inRevisions) {
			revisionKeyArrayHandler.characters(ch, start, length);
		} else {
			buffer.append(ch, start, length);
		}
	}
	
	public Changeset getResult() {
		return result;
	}
	
	private int parseInt(String value) throws SAXException {
		try {
			return Integer.parseInt(value);
		} catch (NumberFormatException e) {
			// TODO fix this
			throw new SAXException("ZOMG");
		}
	}
	
	private Date parseDate(String value) throws SAXException {
		try {
			return format.parse(value);
		} catch (ParseException e) {
			// TODO fix this
			throw new SAXException("ZOMG");
		}
	}
}
